/*
 *  Copyright (C) 2007-2008 CMM Sigma Andrzej Chmielowiec 
<cmmsigma@cmmsigma.eu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

// QtTelnet.cpp: implementation of the QtTelnet class.
//
//////////////////////////////////////////////////////////////////////

#include "QtTelnet.h"

QtTelnet::QtTelnet(QObject *parent) : QObject(parent)
{
	mTcpSocket = NULL;
	createSocket();
}

QtTelnet::~QtTelnet()
{
	delete mTcpSocket;
}


//////////////////////////////////////////////////////////////////////
// public methods
//////////////////////////////////////////////////////////////////////

void QtTelnet::connectToHost(const QString &host, quint16 port)
{
	if(NULL == mTcpSocket)
	{
		createSocket();
	}
	else
	{
		mTcpSocket->abort();
	}

	mTcpSocket->connectToHost(host, port);
}


void QtTelnet::login(const QString &username, const QString &password)
{

}


QTcpSocket *QtTelnet::socket() const
{
	return mTcpSocket;
}


void QtTelnet::close()
{
	mTcpSocket->close();
}


void QtTelnet::logout()
{
	// logout code
	emit loggedOut();
}


void QtTelnet::sendData(QString data)
{
	//tr::trace("* socket send: %s", qPrintable(data));

	data += "\r\n";
	mTcpSocket->write(data.toUtf8());
}

void QtTelnet::sendData(const QStringList &lines)
{
	foreach(QString line, lines)
	{
		//tr::trace("* socket send line: %s", qPrintable(line));

		line += "\r\n";
		mTcpSocket->write(line.toUtf8());
	}
}

//////////////////////////////////////////////////////////////////////
// socket slots
//////////////////////////////////////////////////////////////////////

void QtTelnet::socketConnected()
{
	emit connected();
}

void QtTelnet::socketReadyRead()
{
	if(false == mTcpSocket->canReadLine())
		return;

	QStringList lines;
	QString line;
	do
	{
		line = mTcpSocket->readLine();

		//tr::trace("* socket received line: %s", qPrintable(line));

		lines.append(line);
	} while(mTcpSocket->canReadLine());

	emit message(lines);
/*
	QByteArray buffer = mTcpSocket->readAll();

	QString data = QString(buffer);
	//tr::trace("* socket received: %s", qPrintable(data));
	emit message(data);
*/

/*	QStringList lines = data.split("\r\n", QString::SkipEmptyParts);

	foreach(QString line, lines)
	{
		mResponse.clear();

		processOptions(line);
		respondToOptions();

		if(false == mResponse.isEmpty())
		{
			sendData(mResponse);
		}
	}

	emit message(mNormalText);*/
}

void QtTelnet::socketError(QAbstractSocket::SocketError error)
{
	QString err;

	switch(error)
	{
		case QAbstractSocket::ConnectionRefusedError: err = tr("Poczenie z serwerem zostao przerwane (lub upyn maksymalny czas oczekiwania)."); break;
		case QAbstractSocket::RemoteHostClosedError: err = tr("Serwer niespodziewanie zakoczy poczenie."); break;
		case QAbstractSocket::HostNotFoundError: err = tr("Serwer nie zosta odnaleziony (lub bdnie podany adres serwera)."); break;
		case QAbstractSocket::SocketAccessError: err = tr("Gniazdo sieciowe nie moe wykona zadanej operacji z powodu braku uprawnie."); break;
		case QAbstractSocket::SocketResourceError: err = tr("Brakuje zasobw lokalnych na tym komputerze (np. za duo gniazd sieciowych jest jednoczenie uyciu)."); break;
		case QAbstractSocket::SocketTimeoutError: err = tr("Przekroczony czas oczekiwania na operacj gniazda sieciowego."); break;
		case QAbstractSocket::DatagramTooLargeError: err = tr("Pakiet danych jest wikszy ni dozwolony na tym systemie operacyjnym (ktry nie moe by wikszy ni 8192 bajtw)."); break;
		case QAbstractSocket::NetworkError: err = tr("Bd poczenia sieciowego (np. kabel sieciowy zosta przypadkowo odczony)."); break;
		case QAbstractSocket::AddressInUseError: err = tr("Adres podany w QUdpSocket::bind() jest w uyciu i jest ustawiony w tryb wycznoci."); break;
		case QAbstractSocket::SocketAddressNotAvailableError: err = tr("Adres podany w QUdpSocket::bind() nie naley do hosta."); break;
		case QAbstractSocket::UnsupportedSocketOperationError: err = tr("Operacja na gniedzie nie jest wspierana na tym systemie operacyjnym (np. brak wsparcia dla protokou IPv6)."); break;
		case QAbstractSocket::ProxyAuthenticationRequiredError: err = tr("Gniadzo sieciowe uywa proxy, ktry wymaga uwieytelnienia."); break;
		default: err = tr("Wystpi niezidentyfikowany bd."); break;
	}
	
	mTcpSocket->abort();

	emit connectionError(err);
}

//////////////////////////////////////////////////////////////////////
// protected methods
//////////////////////////////////////////////////////////////////////

void QtTelnet::createSocket()
{
	if(NULL != mTcpSocket)
	{
		mTcpSocket->abort();
		delete mTcpSocket;
	}
	
	mTcpSocket = new QTcpSocket(this);
	connect(mTcpSocket, SIGNAL(readyRead()), this, SLOT(socketReadyRead()));
	connect(mTcpSocket, SIGNAL(connected()), this, SLOT(socketConnected()));
	connect(mTcpSocket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(socketError(QAbstractSocket::SocketError)));
}


void QtTelnet::deleteSocket()
{
	if(NULL != mTcpSocket)
	{
		mTcpSocket->abort();
		delete mTcpSocket;
	}
}


void QtTelnet::processOptions(QString line)
{
	int ndx, ldx;
	QChar ch;
	bool scanDone = false;
	QString option;

	while(!line.isEmpty() && !scanDone)
	{
		ndx = line.indexOf(QChar(IAC));
		if(ndx != -1)
		{
			mNormalText += line.left(ndx);
			ch = line[ndx + 1];
			switch((int)ch.toAscii())
			{
				case DO:
				case DONT:
				case WILL:
				case WONT:
					option = line.mid(ndx, 3);
					line = line.mid(ndx + 3);
					mNormalText = line.left(ndx);
					mListOptions.append(option);
					break;
				case IAC:
					mNormalText = line.left(ndx);
					line = line.mid(ndx + 1);
					break;
				case SB:
					mNormalText = line.left(ndx);
					ldx = line.indexOf(QChar(SE));
					option = line.mid(ndx, ldx);
					mListOptions.append(option);
					line = line.mid(ldx);
					break;
			}
		}

		else
		{
			mNormalText = line;
			scanDone = true;
		}
	}
}


void QtTelnet::respondToOptions()
{
	QString option;
	
	while(!mListOptions.isEmpty())
	{
		option = mListOptions[0];

		mListOptions.removeFirst();

		arrangeReply(option);
	}
}


void QtTelnet::arrangeReply(QString option)
{
	QChar modifier, ch;
	bool defined = false;

	if(option.size() < 3)
		return;

	QChar verb = option[1];
	QChar value = option[2];

	switch((int)value.toAscii())
	{
		case 1:	// Echo
		case 3: // Suppress Go-Ahead
			defined = true;
			break;
	}

	mResponse += QChar(IAC);

	if(defined == true)
	{
		switch((int)verb.toAscii())
		{
			case DO:
				ch = QChar(WILL);
				mResponse += ch;
				mResponse += value;
				break;
			case DONT:
				ch = QChar(WONT);
				mResponse += ch;
				mResponse += value;
				break;
			case WILL:
				ch = QChar(DO);
				mResponse += ch;
				mResponse += value;
				break;
			case WONT:
				ch = QChar(DONT);
				mResponse += ch;
				mResponse += value;
				break;
			case SB:
				modifier = option[3];
				if(modifier == QChar(TELQUAL_SEND))
				{
					ch = QChar(SB);
					mResponse += ch;
					mResponse += option;
					mResponse += QChar(TELQUAL_IS);
					mResponse += QChar(IAC);
					mResponse += QChar(SE);
				}
				break;
		}
	}
	else
	{
		switch((int)value.toAscii())
		{
			case DO:
				ch = QChar(WONT);
				mResponse += ch;
				mResponse += value;
				break;
			case DONT:
				ch = QChar(WONT);
				mResponse += ch;
				mResponse += value;
				break;
			case WILL:
				ch = QChar(DONT);
				mResponse += ch;
				mResponse += value;
				break;
			case WONT:
				ch = QChar(DONT);
				mResponse += ch;
				mResponse += value;
				break;
		}
	}
}
