/*
 *  Copyright (C) 2007-2008 CMM Sigma Andrzej Chmielowiec <cmmsigma@cmmsigma.eu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include "MainWindow.h"
#include "AdminOptionsDialog.h"
#include "SendMailDialog.h"
#include "AboutBox.h"

static const QString welcomeString = QObject::tr(
	"<center><h2>sToken Admin v1.1</h2></center>"
);

/*// QTreeViewExt class
QTreeViewExt::QTreeViewExt(QWidget *parent) : QTreeView(parent)
{
}

void QTreeViewExt::currentChanged(const QModelIndex &current, const QModelIndex &previous)
{
	emit currentIndexChanged(current, previous);
}*/

// MainWindow class

MainWindow *MainWindow::mainWnd = 0;

MainWindow::MainWindow(QMainWindow *parent): QMainWindow(parent)
{
	mainWnd = this;

	mConfDirPath = QDir::homePath() + QDir::separator() + ".stoken_admin";
	mXmlConfFileName = QString("xml_conf");
	
	readXmlConfig();
	
	mFontName = "Helvetica";
	mFontSize = 10;
	setFont(QFont(mFontName, mFontSize));
	
	mModel = new TreeModel(this);
	
	connect(mModel, SIGNAL(pageActivated(QWidget*)), this, 
		SLOT(setActivePage(QWidget*)));
	connect(mModel, SIGNAL(pageDeleted(QWidget*)), this, 
		SLOT(deletePage(QWidget*)));
	
	mView = new QTreeView(this);
	mView->setModel(mModel);
	
	connect(mView, SIGNAL(clicked(const QModelIndex&)), mModel, 
		SLOT(setActive(const QModelIndex&)));
	
	refresh();
	
/*	connect(mView, SIGNAL(currentIndexChanged(const QModelIndex &, const QModelIndex &)), mModel, 
		SLOT(setActive(const QModelIndex&, const QModelIndex &)));

	mView->setContextMenuPolicy(Qt::CustomContextMenu);
	connect(mView, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(showPop()));*/
	
	createActions();
	createMenus();
	createToolBars();
	
	mSplitter = new QSplitter;
	mPages = new QStackedWidget;
	
	QGridLayout *grid = new QGridLayout;
	mMainPage = new QWidget();
	mMainPage->setLayout(grid);
	mPages->addWidget(mMainPage);
	
	QLabel *label = new QLabel();
	label->setPixmap(QPixmap(QString(":img/stoken_logo_120_60.png")));
	label->setAlignment(Qt::AlignRight);
	grid->addWidget(label, 0, 0);
	grid->addWidget(new QLabel(welcomeString, this), 1, 0);
	grid->setRowStretch(1, 1);
	grid->setRowStretch(2, 1);
	
	mSplitter->addWidget(mView);
	mSplitter->addWidget(mPages);
	
	QList<int> sizes;
	sizes.append(200);
	sizes.append(600);
	mSplitter->setSizes(sizes);
	mSplitter->setStretchFactor(1, 1);
//	mSplitter->setMinimumSize(600, 300);
	
	setCentralWidget(mSplitter);
	
	setWindowIcon(QIcon(QString(":img/stoken_admin_032.png")));
	setWindowTitle(tr("sToken Admin"));
	
	resize(750, 600);
}

void
MainWindow::setActivePage(QWidget *page)
{
	mPages->removeWidget(mPages->currentWidget());
	mPages->addWidget(page);
}

void
MainWindow::deletePage(QWidget *page)
{
	if (mPages->currentWidget() == page)
		setActivePage(mMainPage);
}

void
MainWindow::incFontSize()
{
	if (mFontSize < 24)
	{
		setFont(QFont(mFontName, ++mFontSize));
		mDecFontSizeAct->setEnabled(true);
		
		if (mFontSize == 24)
			mIncFontSizeAct->setEnabled(false);
	}
}

void
MainWindow::decFontSize()
{
	if (8 < mFontSize)
	{
		mIncFontSizeAct->setEnabled(true);
		setFont(QFont(mFontName, --mFontSize));
		
		if (mFontSize == 8)
			mDecFontSizeAct->setEnabled(false);
	}
}

void
MainWindow::createActions()
{
	mExitAct = new QAction(QIcon(QString(":img/exit.png")), tr("Zakocz"), this);
	mExitAct->setShortcut(tr("Ctrl+Q"));
	connect(mExitAct, SIGNAL(triggered()), this, SLOT(close()));
	
	mSettingsAct = new QAction(QIcon(QString(":img/configure.png")), tr("Konfiguracja"), this);
	connect(mSettingsAct, SIGNAL(triggered()), this, SLOT(settings()));
	mSettingsAct->setShortcut(tr("F11"));
	
	mRefreshAct = new QAction(QIcon(QString(":img/refresh.png")), tr("Odwie"), this);
	mRefreshAct->setShortcut(tr("F5"));
	connect(mRefreshAct, SIGNAL(triggered()), this, SLOT(refresh()));
	
	mIncFontSizeAct = new QAction(QIcon(QString(":img/view_plus.png")), tr("Powiksz"), this);
	mIncFontSizeAct->setShortcut(tr("Ctrl++"));
	connect(mIncFontSizeAct, SIGNAL(triggered()), this, SLOT(incFontSize()));
	
	mDecFontSizeAct = new QAction(QIcon(QString(":img/view_minus.png")), tr("Pomniejsz"), this);
	mDecFontSizeAct->setShortcut(tr("Ctrl+-"));
	connect(mDecFontSizeAct, SIGNAL(triggered()), this, SLOT(decFontSize()));
	
	mHelpAct = new QAction(QIcon(QString(":img/contents_ii.png")), tr("Podrcznik"), this);
	mHelpAct->setShortcut(tr("Ctrl+H"));
	
	mAboutAct = new QAction(QIcon(QString(":img/info.png")), tr("O programie"), this);
	connect(mAboutAct, SIGNAL(triggered()), this, SLOT(aboutSlot()));
}

void
MainWindow::createMenus()
{
	mGeneralMenu = menuBar()->addMenu(tr("Oglne"));
	mGeneralMenu->addAction(mExitAct);
	
	mGeneralMenu = menuBar()->addMenu(tr("Ustawienia"));
	mGeneralMenu->addAction(mSettingsAct);
	
	mViewMenu = menuBar()->addMenu(tr("Widok"));
	mViewMenu->addAction(mRefreshAct);
	mViewMenu->addSeparator();
	mViewMenu->addAction(mIncFontSizeAct);
	mViewMenu->addAction(mDecFontSizeAct);
	
	mHelpMenu = menuBar()->addMenu(tr("Pomoc"));
	mHelpMenu->addAction(mHelpAct);
	mHelpMenu->addSeparator();
	mHelpMenu->addAction(mAboutAct);
}

void
MainWindow::createToolBars()
{
	mViewToolBar = addToolBar(tr("Widok"));
	mViewToolBar->addAction(mRefreshAct);
	mViewToolBar->addAction(mIncFontSizeAct);
	mViewToolBar->addAction(mDecFontSizeAct);
	
	mHelpToolBar = addToolBar(tr("Pomoc"));
	mHelpToolBar->addAction(mHelpAct);
	mHelpToolBar->addAction(mAboutAct);
}

void
MainWindow::showPop()
{
	mModel->execContextMenu(mView->currentIndex());
}

void
MainWindow::settings()
{
	AdminOptionsDialog *dialog = new AdminOptionsDialog(this);
	dialog->setFont(QFont(mFontName, mFontSize));
	dialog->setWindowTitle(QObject::tr("Konfiguracja"));

	readXmlConfig();
	dialog->setData(config);
	
	if (dialog->exec() == QDialog::Accepted)
	{
		dialog->getData(config);
		writeXmlConfig();
	}
}

bool
MainWindow::writeXmlConfig()
{
	QFile file(mConfDirPath + QDir::separator() + mXmlConfFileName);
	if(file.open(QIODevice::WriteOnly))
	{
		config.serialize(file);
		file.close();
	}
	return true;
}

bool
MainWindow::readXmlConfig()
{
	QFile file(mConfDirPath + QDir::separator() + mXmlConfFileName);
	if(file.open(QIODevice::ReadOnly))
	{
		config.deserialize(file);
		file.close();
	}
	return true;
}

AdminOptions *
MainWindow::getConfig()
{
	return &config;
}

void
MainWindow::sendRequest(const QString &fileName, const QString &mail)
{
	if(!config.mailConfig.checkOutgoingConfig())
	{
		QMessageBox::information(this, tr("Informacja"), tr("Wysyanie "
			"zgosze, wymaga poprawnie skonfigurowanego konta pocztowego."));

		return;
	}

	SendMailDialog dlg(this);
	QtSmtp *smtp = dlg.getSmtp();

	smtp->beginNewMail();
	smtp->setServer(config.mailConfig.mSmtpServer);
	smtp->setSender(config.mailConfig.mMail, config.mailConfig.mUser);

	smtp->addRecipient(mail);
	smtp->setMessage(tr("To jest wiadomosc zawierajaca zgloszenie."));
	smtp->setSubject(tr("Zgloszenie"));

	if(config.mailConfig.mOutgoingAuth)
	{
		if(config.mailConfig.mUseIncomming)
			smtp->setUserPassword(config.mailConfig.mPop3User , config.mailConfig.mPop3Pass);
		else
			smtp->setUserPassword(config.mailConfig.mSmtpUser , config.mailConfig.mSmtpPass);
	}

	smtp->addFileAttachment(fileName);

	smtp->sendMail();
	dlg.exec();
	
	QFile::remove(fileName);
}

void
MainWindow::aboutSlot()
{
	AboutBox dlg(this);
	dlg.exec();
}

void
MainWindow::refresh()
{
	mModel->refresh();
	mView->setIconSize(QSize(16,16));
	mView->setIconSize(QSize(24,24));
	mView->repaint();
}
