/*
 *  Copyright (C) 2007-2008 CMM Sigma Andrzej Chmielowiec <cmmsigma@cmmsigma.eu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include "CaNewDialog.h"
#include "STokenDialog.h"

#include <QtGui>

CaNewDialog::CaNewDialog(QWidget *parent, Qt::WindowFlags f): QDialog(parent, f)
{
	QGridLayout *layout = new QGridLayout();
	
	mailWidget = new MailAccountWidget(true, true, this);

	QTabWidget *tab = new QTabWidget();
	tab->addTab(createGeneralTab(), tr("Oglne"));
	tab->addTab(createCaSubjectTab(), tr("Dane urzdu"));
	tab->addTab(createPolicyTab(), tr("Polityka urzdu"));
	tab->addTab(mailWidget, tr("Konto pocztowe"));
	
	mCancel = new QPushButton(tr("A&nuluj"));
	connect(mCancel, SIGNAL(clicked()), this, SLOT(reject()));
	mAccept = new QPushButton(tr("U&twrz"));
	connect(mAccept, SIGNAL(clicked()), this, SLOT(accept()));
	
	layout->addWidget(tab, 0, 0, 1, 3);
	layout->addWidget(mAccept, 1, 1);
	layout->addWidget(mCancel, 1, 2);
	
	layout->setColumnStretch(0, 1);
	layout->setRowStretch(0, 1);
	
	setLayout(layout);
}

QString
CaNewDialog::name()
{
	return mName->text();
}

QString
CaNewDialog::caKeyName()
{
	return mCaKeyName->text();
}

int
CaNewDialog::defaultCertDays()
{
	return mDefaultCertDays->value();
}

int
CaNewDialog::defaultCrlDays()
{
	return mDefaultCrlDays->value();
}

QString
CaNewDialog::defaultMD()
{
	QComboBox *combo = mDefaultMD;
	return combo->itemData(combo->currentIndex()).toString();
}

int
CaNewDialog::caCertDays()
{
	return mCaCertDays->value();
}

QString
CaNewDialog::caSubject()
{
	QString subject;
	
	subject.append("/C=").append(mCaCountryName->currentText());
	
	if (mCaStateOrProvinceName->text() != QString())
		subject.append("/ST=").append(mCaStateOrProvinceName->text());
	
	if (mCaLocalityName->text() != QString())
		subject.append("/L=").append(mCaLocalityName->text());
	
	if (mCaOrganizationName->text() != QString())
		subject.append("/O=").append(mCaOrganizationName->text());
	
	if (mCaOrganizationalUnitName->text() != QString())
		subject.append("/OU=").append(mCaOrganizationalUnitName->text());
	
	if (mCaCommonName->text() != QString())
		subject.append("/CN=").append(mCaCommonName->text());
	
	if (mCaEmailAddress->text() != QString())
		subject.append("/emailAddress=").append(mCaEmailAddress->text());
	
	return subject;
}

QString
CaNewDialog::policyCountryName()
{
	QComboBox *combo = mPolicyCountryName;
	return combo->itemData(combo->currentIndex()).toString();
}

QString
CaNewDialog::policyStateOrProvinceName()
{
	QComboBox *combo = mPolicyStateOrProvinceName;
	return combo->itemData(combo->currentIndex()).toString();
}

QString
CaNewDialog::policyLocalityName()
{
	QComboBox *combo = mPolicyLocalityName;
	return combo->itemData(combo->currentIndex()).toString();
}

QString
CaNewDialog::policyOrganizationName()
{
	QComboBox *combo = mPolicyOrganizationName;
	return combo->itemData(combo->currentIndex()).toString();
}

QString
CaNewDialog::policyOrganizationalUnitName()
{
	QComboBox *combo = mPolicyOrganizationalUnitName;
	return combo->itemData(combo->currentIndex()).toString();
}

QString
CaNewDialog::policyCommonName()
{
	QComboBox *combo = mPolicyCommonName;
	return combo->itemData(combo->currentIndex()).toString();
}

QString
CaNewDialog::policyEmailAddress()
{
	QComboBox *combo = mPolicyEmailAddress;
	return combo->itemData(combo->currentIndex()).toString();
}

void
CaNewDialog::accept()
{
	if (mName->text().isEmpty())
	{
		QMessageBox::information(this, tr("Brak danych"), tr("Wprowad nazw urzdu."));
		return;
	}
	
#ifndef STOKEN_SOFT
	if (mCaKeyName->text().isEmpty())
	{
		QMessageBox::information(this, tr("Brak danych"), tr("Wprowad nazw klucza "
			"urzdu."));
		return;
	}
#endif
	
	if (mCaCommonName->text().isEmpty())
	{
		QMessageBox::information(this, tr("Brak danych"), tr("Wprowad nazw pospolit "
			"urzdu (to pole jest wymagane)."));
		return;
	}
	
	if (QDir(QCoreApplication::applicationDirPath() + QString("/conf/") + mName->text()).exists())
	{
		QMessageBox::information(this, QObject::tr("Informacja"), QObject::tr("Urzd o "
			"podanej nazwie ju istnieje. Musisz poda inn nazw."));
		return;
	}
	
	QDialog::accept();
}

void
CaNewDialog::browseCaKeyName()
{
	QString keyName;
	
	keyName = STokenDialog::getKeyName(tr("Wybr klucza"), this);
	
	if (keyName != QString())
		mCaKeyName->setText(keyName);
}

void
CaNewDialog::nameCheck(const QString &name)
{
	for (int i = 0; i < name.size(); i++)
	{
		if ( ((name[i] < QChar('a')) || (QChar('z') < name[i]))
			&& ((name[i] < QChar('A')) || (QChar('Z') < name[i]))
			&& ((name[i] < QChar('0')) || (QChar('9') < name[i]))
			&& (name[i] != QChar('_')) && (name[i] != QChar('-')) )
		{
			QMessageBox::information(this, tr("Niepoprawne dane"), 
				tr("Pole nazwa urzdu moze zawiera jedynie znaki "
				"alfanumeryczne [a-z][A-Z][0-9] oraz '_' i '-'."));
			mName->setText(mCorrectName);
		}
	}
	
	mCorrectName = mName->text();
}

QWidget*
CaNewDialog::createGeneralTab()
{
	QWidget *widget = new QWidget();
	QGridLayout *layout = new QGridLayout();
	
	widget->setLayout(layout);

#ifndef STOKEN_SOFT
	QGroupBox *box = new QGroupBox(tr("Nazwa i klucz urzdu"));
#else
	QGroupBox *box = new QGroupBox(tr("Nazwa urzdu"));
#endif
	QGridLayout *boxLayout = new QGridLayout();
	
	box->setLayout(boxLayout);
	
	mName = new QLineEdit();
	connect(mName, SIGNAL(textChanged(const QString&)), this, 
		SLOT(nameCheck(const QString&)));
	mCaKeyName = new QLineEdit();
#ifndef STOKEN_SOFT
	mCaKeyBrowse = new QPushButton(tr("Wybierz"));
	connect(mCaKeyBrowse, SIGNAL(clicked()), this, SLOT(browseCaKeyName()));
	boxLayout->addWidget(new QLabel(tr("Klucz urzdu<sup>*</sup>")), 1, 0);
	boxLayout->addWidget(mCaKeyName, 1, 1, 1, 2);
	boxLayout->addWidget(mCaKeyBrowse, 2, 2);
#endif
	boxLayout->addWidget(new QLabel(tr("Nazwa urzdu<sup>*</sup>")), 0, 0);
	boxLayout->addWidget(mName, 0, 1, 1, 2);
	boxLayout->setColumnStretch(1, 1);
	
	layout->addWidget(box, 0, 0);
	
	box = new QGroupBox(tr("Ustawienia domylne"));
	boxLayout = new QGridLayout();
	
	box->setLayout(boxLayout);
	
	mDefaultCertDays = new QSpinBox();
	mDefaultCertDays->setMinimum(1);
	mDefaultCertDays->setMaximum(20 * 365);
	mDefaultCertDays->setValue(365);
	mDefaultCertDays->setSuffix(QString(" [dni]"));
	
	mDefaultCrlDays = new QSpinBox();
	mDefaultCrlDays->setMinimum(1);
	mDefaultCrlDays->setMaximum(183);
	mDefaultCrlDays->setValue(30);
	mDefaultCrlDays->setSuffix(QString(" [dni]"));
	
	mDefaultMD = new QComboBox();
	mDefaultMD->addItem(QString("SHA-1"), QVariant(QString("ecdsa-with-sha1")));
	mDefaultMD->addItem(QString("SHA-224"), QVariant(QString("ecdsa-with-sha224")));
	mDefaultMD->addItem(QString("SHA-256"), QVariant(QString("ecdsa-with-sha256")));
	mDefaultMD->setCurrentIndex(2);
	
	boxLayout->addWidget(new QLabel(tr("Okres wanoci certyfiaktu uytkownika")), 0, 0);
	boxLayout->addWidget(mDefaultCertDays, 0, 1, 1, 2);
	boxLayout->addWidget(new QLabel(tr("Okres wanoci listy CRL")), 1, 0);
	boxLayout->addWidget(mDefaultCrlDays, 1, 1, 1, 2);
	boxLayout->addWidget(new QLabel(tr("Funkcja skrtu")), 2, 0);
	boxLayout->addWidget(mDefaultMD, 2, 1, 1, 2);
	boxLayout->setColumnStretch(1, 1);
	
	layout->addWidget(box, 1, 0);
	
	layout->setRowStretch(2, 1);
	
	return widget;
}

QWidget*
CaNewDialog::createCaSubjectTab()
{
	QWidget *widget = new QWidget();
	QGridLayout *layout = new QGridLayout();
	
	widget->setLayout(layout);
	
	QGroupBox *box = new QGroupBox(tr("Wano certyfikatu urzdu"));
	QGridLayout *boxLayout = new QGridLayout();
	
	box->setLayout(boxLayout);
	
	mCaCertDays = new QSpinBox();
	mCaCertDays->setMinimum(1);
	mCaCertDays->setMaximum(20 * 365);
	mCaCertDays->setValue(20 * 365);
	mCaCertDays->setSuffix(QString(" [dni]"));
	
	boxLayout->addWidget(new QLabel(tr("Okres wanoci certyfiaktu")), 0, 0);
	boxLayout->addWidget(mCaCertDays, 0, 1);
	
	boxLayout->setColumnStretch(1, 1);
	layout->addWidget(box, 0, 0);
	
	box = new QGroupBox(tr("Informacje o urzdzie"));
	boxLayout = new QGridLayout();
	
	box->setLayout(boxLayout);
	
	boxLayout->addWidget(new QLabel(tr("Kod kraju")), 0, 0);
	mCaCountryName = new QComboBox();
	mCaCountryName->addItem(tr("PL"));
	boxLayout->addWidget(mCaCountryName, 0, 1);
	
	boxLayout->addWidget(new QLabel(tr("Region")), 1, 0);
	mCaStateOrProvinceName = new QLineEdit();
	boxLayout->addWidget(mCaStateOrProvinceName, 1, 1);
	
	boxLayout->addWidget(new QLabel(tr("Miejscowo")), 2, 0);
	mCaLocalityName = new QLineEdit();
	boxLayout->addWidget(mCaLocalityName, 2, 1);
	
	boxLayout->addWidget(new QLabel(tr("Organizacja")), 3, 0);
	mCaOrganizationName = new QLineEdit();
	boxLayout->addWidget(mCaOrganizationName, 3, 1);
	
	boxLayout->addWidget(new QLabel(tr("Dzia")), 4, 0);
	mCaOrganizationalUnitName = new QLineEdit();
	boxLayout->addWidget(mCaOrganizationalUnitName, 4, 1);
	
	boxLayout->addWidget(new QLabel(tr("Nazwa pospolita<sup>*</sup>")), 5, 0);
	mCaCommonName = new QLineEdit();
	connect(mCaCommonName, SIGNAL(textChanged(const QString &)),
		this, SLOT(commonChanged(const QString &)));
	boxLayout->addWidget(mCaCommonName, 5, 1);
	
	boxLayout->addWidget(new QLabel(tr("E-mail")), 6, 0);
	mCaEmailAddress = new QLineEdit();
	connect(mCaEmailAddress, SIGNAL(textChanged(const QString &)),
		this, SLOT(emailChanged(const QString &)));
	boxLayout->addWidget(mCaEmailAddress, 6, 1);
	
	boxLayout->setColumnStretch(1, 1);
	layout->addWidget(box, 1, 0);
	
	layout->addWidget(new QLabel(), 2, 0);
	layout->setRowStretch(2, 1);
	
	return widget;
}

QWidget*
CaNewDialog::createPolicyTab()
{
	QWidget *widget = new QWidget();
	QGridLayout *layout = new QGridLayout();
	
	widget->setLayout(layout);
	
	QGroupBox *box = new QGroupBox(tr("Informacje o certyfikowanym podmiocie"));
	QGridLayout *boxLayout = new QGridLayout();
	
	box->setLayout(boxLayout);
	
	boxLayout->addWidget(new QLabel(tr("Kod kraju")), 0, 0);
	mPolicyCountryName = new QComboBox();
	mPolicyCountryName->addItem(tr("opcjonalne"), QVariant(QString("optional")));
	mPolicyCountryName->addItem(tr("wymagane"), QVariant(QString("supplied")));
	mPolicyCountryName->addItem(tr("znaczce"), QVariant(QString("match")));
	boxLayout->addWidget(mPolicyCountryName, 0, 1);
	
	boxLayout->addWidget(new QLabel(tr("Region")), 1, 0);
	mPolicyStateOrProvinceName = new QComboBox();
	mPolicyStateOrProvinceName->addItem(tr("opcjonalne"), QVariant(QString("optional")));
	mPolicyStateOrProvinceName->addItem(tr("wymagane"), QVariant(QString("supplied")));
	mPolicyStateOrProvinceName->addItem(tr("znaczce"), QVariant(QString("match")));
	boxLayout->addWidget(mPolicyStateOrProvinceName, 1, 1);
	
	boxLayout->addWidget(new QLabel(tr("Miejscowo")), 2, 0);
	mPolicyLocalityName = new QComboBox();
	mPolicyLocalityName->addItem(tr("opcjonalne"), QVariant(QString("optional")));
	mPolicyLocalityName->addItem(tr("wymagane"), QVariant(QString("supplied")));
	mPolicyLocalityName->addItem(tr("znaczce"), QVariant(QString("match")));
	boxLayout->addWidget(mPolicyLocalityName, 2, 1);
	
	boxLayout->addWidget(new QLabel(tr("Organizacja")), 3, 0);
	mPolicyOrganizationName = new QComboBox();
	mPolicyOrganizationName->addItem(tr("opcjonalne"), QVariant(QString("optional")));
	mPolicyOrganizationName->addItem(tr("wymagane"), QVariant(QString("supplied")));
	mPolicyOrganizationName->addItem(tr("znaczce"), QVariant(QString("match")));
	boxLayout->addWidget(mPolicyOrganizationName, 3, 1);
	
	boxLayout->addWidget(new QLabel(tr("Dzia")), 4, 0);
	mPolicyOrganizationalUnitName = new QComboBox();
	mPolicyOrganizationalUnitName->addItem(tr("opcjonalne"), QVariant(QString("optional")));
	mPolicyOrganizationalUnitName->addItem(tr("wymagane"), QVariant(QString("supplied")));
	mPolicyOrganizationalUnitName->addItem(tr("znaczce"), QVariant(QString("match")));
	boxLayout->addWidget(mPolicyOrganizationalUnitName, 4, 1);
	
	boxLayout->addWidget(new QLabel(tr("Nazwa pospolita")), 5, 0);
	mPolicyCommonName = new QComboBox();
	mPolicyCommonName->addItem(tr("wymagane"), QVariant(QString("supplied")));
	boxLayout->addWidget(mPolicyCommonName, 5, 1);
	
	boxLayout->addWidget(new QLabel(tr("E-mail")), 6, 0);
	mPolicyEmailAddress = new QComboBox();
	mPolicyEmailAddress->addItem(tr("opcjonalne"), QVariant(QString("optional")));
	mPolicyEmailAddress->addItem(tr("wymagane"), QVariant(QString("supplied")));
	mPolicyEmailAddress->addItem(tr("znaczce"), QVariant(QString("match")));
	boxLayout->addWidget(mPolicyEmailAddress, 6, 1);
	
	boxLayout->setColumnStretch(1, 1);
	
	layout->addWidget(box, 0, 0);
	layout->setRowStretch(1, 1);
	
	return widget;
}

MailAccountWidget *CaNewDialog::getMailWidget()
{
	return mailWidget;
}

void
CaNewDialog::commonChanged(const QString &common)
{
	mailWidget->setUser(common);
}

void
CaNewDialog::emailChanged(const QString &email)
{
	mailWidget->setEmail(email);
}
