/*
 *  Copyright (C) 2007-2008 CMM Sigma Andrzej Chmielowiec <cmmsigma@cmmsigma.eu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include "MainWindow.h"
#include "FileDialogWPR.h"
#include "GenCertReqWidget.h"
#include "QDetailsMessageBox.h"
#include "AboutBox.h"

// MainWindow class
MainWindow::MainWindow()
{
	mConfDirPath = QDir::homePath() + QDir::separator() + ".stoken_softadmin";
	mOpensslConfFile = mConfDirPath + QDir::separator() + "openssl.cnf";
	if(!QFile::exists(mOpensslConfFile))
	{
		QFile srcFile(":openssl.cnf");
		srcFile.open(QIODevice::ReadOnly);
		QByteArray content = srcFile.readAll();
		srcFile.close();

		QFile dstFile(mOpensslConfFile);
		dstFile.open(QIODevice::WriteOnly | QIODevice::Truncate);
		dstFile.write(content);
		dstFile.close();
	}

	QWidget *mainWidget = new QWidget();

	QGridLayout *layout = new QGridLayout();
	mainWidget->setLayout(layout);

	QGroupBox *box = new QGroupBox(tr("Klucz prywatny"));

	QGridLayout *boxLayout = new QGridLayout();
	box->setLayout(boxLayout);

	mAddKey = new QPushButton(QIcon(QString(":img/add.png")), tr("Generuj &nowy klucz"));
	connect(mAddKey, SIGNAL(clicked()), this, SLOT(addKeySlot()));
	mSelectKey = new QPushButton(QIcon(QString(":img/keys_folder_blue.png")), tr("&Wska plik z kluczem"));
	connect(mSelectKey, SIGNAL(clicked()), this, SLOT(selectKeySlot()));

	mKeyPath = new QLineEdit();
	mKeyPath->setReadOnly(true);
	boxLayout->addWidget(new QLabel(tr("Plik klucza:")), 0, 0);
	boxLayout->addWidget(mKeyPath, 0, 1);

	QGridLayout *btnLayout = new QGridLayout();
	boxLayout->addLayout(btnLayout, 1, 0, 1, 0);

	btnLayout->addWidget(mAddKey, 0, 0, Qt::AlignRight);
	btnLayout->addWidget(mSelectKey, 0, 1, Qt::AlignRight);
	btnLayout->setColumnStretch(0, 1);
	
	layout->addWidget(box, 0, 0, Qt::AlignTop);

	QLabel *label = new QLabel();
	label->setPixmap(QPixmap(QString(":img/stoken_logo_120_60.png")));
	label->setAlignment(Qt::AlignRight);
	layout->addWidget(label, 0, 1);


	mGenCert = new GenCertReqWidget(this);
	connect(mGenCert, SIGNAL(generateCertReqSig(QString&, QStringList&, QString&)), 
		this, SLOT(generateCertReqSlot(QString&, QStringList&, QString&)));

	box = new QGroupBox(tr("Zgoszenie certyfikacyjne"));
	boxLayout = new QGridLayout();
	box->setLayout(boxLayout);

	boxLayout->addWidget(mGenCert, 1, 0);
	layout->addWidget(box, 1, 0, 1, 0);

	createActions();
	createMenus();
	//createToolBars();

	setWindowIcon(QIcon(QString(":img/stoken_softadmin_032.png")));
	setWindowTitle(tr("sToken SoftAdmin"));
	setCentralWidget(mainWidget);
	resize(580, 300);

	// openssl command
	mOpensslCmd = QCoreApplication::applicationDirPath().append("/../bin/stoken_openssl");

#	ifdef Q_WS_WIN
	mOpensslCmd.append(QString(".exe"));
#	endif
}

QString
MainWindow::getPIN(bool confirm)
{
	bool ok;
	QString pin1, pin2;
	
	do
	{
		pin1 = QInputDialog::getText(this, tr("Podaj haso do klucza prywatnego"),
			tr("Haso:"), QLineEdit::Password, QString(), &ok);
		if (!ok)
			return QString();

		if(pin1.isEmpty())
			QMessageBox::warning(this, tr("Informacja"), tr("Haso do klucza nie moe by puste."));
	} while(pin1.isEmpty());

	if(confirm)
	{
		pin2 = QInputDialog::getText(this, tr("Podaj powtrzone haso"),
			tr("Powtrzone haso:"), QLineEdit::Password, QString(), &ok);
		if (!ok)
			return QString();

		if(pin1 != pin2)
		{
			QMessageBox::information(this, tr("Informacja"), tr("Podane hasa "
				"nie s jednakowe."));
			return QString();
		}
		else
			return pin1;
	}
	else
		return pin1;
}

void
MainWindow::addKeySlot()
{
	QString keyPath = FileDialogWPR::getSaveFileName(this, tr("Podaj nazw pliku klucza prywatnego"),
		QString(), tr("Pliki klucza (*.key)"));
	
	if(!keyPath.isEmpty())
	{
		QString pin = getPIN(true);
		if(!pin.isEmpty())
		{
			if(true == generateKey(pin, keyPath))
			{
				mKeyPath->setText(keyPath);
				mGenCert->enable();
			}
		}
	}
}

void
MainWindow::selectKeySlot()
{
	QString keyPath = FileDialogWPR::getOpenFileName(this, tr("Wska plik z kluczem prywatnym"),
		QString(), tr("Pliki klucza (*.key);;Wszystkie pliki (*)"));

	if(!keyPath.isEmpty())
	{
		mKeyPath->setText(keyPath);
		mGenCert->enable();
	}
}

bool
MainWindow::runOpenssl()
{
	QProcess openssl;
	
	openssl.setWorkingDirectory(mOpensslWorkDir);
	openssl.start(mOpensslCmd, mOpensslArgs);

	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	
	while (openssl.state() != QProcess::NotRunning)
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
	
	QApplication::restoreOverrideCursor();
	
	mOpensslOut.clear();
	mOpensslError.clear();
	
	mOpensslOut = QString(openssl.readAllStandardOutput());
	mOpensslError = QString(openssl.readAllStandardError());
	
	if (openssl.exitCode() != 0)
		return false;
	
	return true;
}

bool
MainWindow::generateKey(const QString &pin, const QString &keyFile)
{
	QFileInfo info(keyFile);
	mOpensslWorkDir = info.path();
	mOpensslArgs.clear();

	// Run OpenSSL and create key.
	mOpensslArgs << "ecparam" << "-name" << "prime256v1" << "-genkey" << "-pass" << pin <<
		"-aes256" << "-out" << info.filePath();

	if (!runOpenssl())
	{
		QDetailsMessageBox::critical(this, tr("Bd"), tr("Nie mona "
			"utworzy klucza."), mOpensslError);
		return false;
	}
	else
	{
		QMessageBox::information(this, tr("Informacja"), tr("Klucz zosta "
			"utworzony poprawnie."));

		// change permissions of created key file
		QFile::setPermissions(info.filePath(), QFile::ReadUser | QFile::ReadOwner);

		return true;
	}
}

bool
MainWindow::generateCertRequest(QString &fileName, QStringList &dn)
{
	if(mKeyPath->text().isEmpty())
		return false;

	QString pin = getPIN();
	if(pin.isEmpty())
		return false;

	QFileInfo info(fileName);
	mOpensslWorkDir = info.path();
	mOpensslArgs.clear();

	// Run OpenSSL and create request.
	mOpensslArgs << "req" << "-key" << mKeyPath->text() <<
		"-keyform" << "ENGINE" << "-engine" << "stokensoft" << "-engctrl" <<
		QString("user_pin:").append(pin) << "-md" << "ecdsa-with-sha256" << "-new" << "-utf8" <<
		"-batch" << "-config" << mOpensslConfFile << "-out" << fileName <<"-subj";

	QString subject;
	foreach(QString subitem, dn)
	{
		subject += "/";
		subject += subitem;
	}
	mOpensslArgs << subject;

	if (!runOpenssl())
	{
		if(mOpensslError.contains("error:0906A065"))
		{
			QDetailsMessageBox::critical(this, tr("Bd"), tr("Podane haso jest nieprawidowe."),
				mOpensslError);
		}
		else
		{
			QDetailsMessageBox::critical(this, tr("Bd"), tr("Nie mona "
				"utworzy zgoszenia certyfikatu."), mOpensslError);
		}
		return false;
	}
	else
	{
		QMessageBox::information(this, tr("Informacja"), tr("Zgoszenie certyfikatu zostao"
			" utworzone poprawnie."));
		return true;
	}

	return true;
}

void
MainWindow::generateCertReqSlot(QString &fileName, QStringList &dn, QString &mail)
{
	generateCertRequest(fileName, dn);
}

void
MainWindow::createActions()
{
	mExitAct = new QAction(QIcon(QString(":img/exit.png")), tr("Zakocz"), this);
	mExitAct->setShortcut(tr("Ctrl+Q"));
	connect(mExitAct, SIGNAL(triggered()), this, SLOT(close()));
	
	mAddKeyAct = new QAction(QIcon(QString(":img/add.png")), tr("Generuj nowy klucz"), this);
	connect(mAddKeyAct, SIGNAL(triggered()), this, SLOT(addKeySlot()));
	
	mSelectKeyAct = new QAction(QIcon(QString(":img/keys_folder_blue.png")), tr("Wska plik z kluczem"), this);
	connect(mSelectKeyAct, SIGNAL(triggered()), this, SLOT(selectKeySlot()));
	
	mHelpAct = new QAction(QIcon(QString(":img/contents_ii.png")), tr("Podrcznik"), this);
	mHelpAct->setShortcut(tr("Ctrl+H"));
	
	mAboutAct = new QAction(QIcon(QString(":img/info.png")), tr("O programie"), this);
	connect(mAboutAct, SIGNAL(triggered()), this, SLOT(aboutSlot()));
}

void
MainWindow::createMenus()
{
	QMenu *menu;

	menu = menuBar()->addMenu(tr("&Oglne"));
	menu->addAction(mExitAct);
	
	menu = menuBar()->addMenu(tr("&Klucz"));
	menu->addAction(mAddKeyAct);
	menu->addAction(mSelectKeyAct);
	
	menu = menuBar()->addMenu(tr("&Pomoc"));
	menu->addAction(mHelpAct);
	menu->addSeparator();
	menu->addAction(mAboutAct);
}

void
MainWindow::createToolBars()
{
	QToolBar *toolBar;
	toolBar = addToolBar(tr("Klucz"));
	toolBar->addAction(mAddKeyAct);
	toolBar->addAction(mSelectKeyAct);
	
	toolBar = addToolBar(tr("Pomoc"));
	toolBar->addAction(mHelpAct);
	toolBar->addAction(mAboutAct);
}

void
MainWindow::aboutSlot()
{
	AboutBox dlg(this);
	dlg.exec();
}
