/*
 *  Copyright (C) 2007-2008 CMM Sigma Andrzej Chmielowiec <cmmsigma@cmmsigma.eu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#ifndef MAIN_WINDOW_H
#define MAIN_WINDOW_H

#include <QtGui>
#include <QMainWindow>

class GenCertReqWidget;

class MainWindow: public QMainWindow
{
	Q_OBJECT
	
public:
	MainWindow();
	
private:
	QLineEdit *mKeyPath;
	QPushButton *mAddKey;
	QPushButton *mSelectKey;
	GenCertReqWidget *mGenCert;

	// OpenSSL command name and arguments.
	QString mOpensslCmd;
	QStringList mOpensslArgs;
	QString mConfDirPath;
	QString mOpensslConfFile;
	QString mOpensslOut;
	QString mOpensslError;
	QString mOpensslWorkDir;

	bool runOpenssl();
	bool generateKey(const QString &pin, const QString &keyFile);
	bool generateCertRequest(QString &fileName, QStringList &dn);

	void createActions();
	void createMenus();
	void createToolBars();

	QString getPIN(bool confirm = false);
	
	QAction *mExitAct;
	QAction *mAddKeyAct;
	QAction *mSelectKeyAct;
	QAction *mHelpAct;
	QAction *mAboutAct;
	
private slots:
	void addKeySlot();
	void selectKeySlot();
	void generateCertReqSlot(QString &fileName, QStringList &dn, QString &mail);
	void aboutSlot();
};

#endif
